class Content(object):

    def __init__(self, content_id=None, clicks=0, site=None, **kwargs):
        """creates a new instance


        :param content_id: the content's unique id
        :type content_id: str or int

        :param clicks: the number of clicks for the content
        :type clicks: int

        :param site: the site's name
        :type site: str or None
        :default site: None

        :param kwargs: additional keyword arguments that aren't really used, but whatever i'll keep them around
        """
        self.content_id = content_id
        self.clicks = clicks
        self.site = site
        for key, value in kwargs.items():
            setattr(self, '_{}'.format(key), value)

    def __str__(self):
        return '[{}] {}: {}'.format(self.site, self.content_id, self.clicks)

    def __lt__(self, other):
        return self.clicks < other.clicks

    @classmethod
    def from_query_result(cls, columns, point):
        """creates an instance of Content from pieces of an influxdb query


        :param columns: the list of column names (keys)
        :type columns: [str, ]

        :param point: the list of associated instance values for the columns
        :type point: [type, ]

        :return: a new instance of Content
        :rtype: Content
        """
        return Content(**dict(zip(columns, point)))



from influxdb import InfluxDBClient

from .models import Content


class ContentClient(object):
    """a client object to interact with influxdb
    """

    def __init__(self, host, port, username, password, db, series):
        """creates a new instance


        :param host: the hostname or ip address to connect the influxdb client
        :type host: str

        :param port: the port number to connect over
        :type port: int

        :param username: a read/write user's username
        :type username: str

        :param password: a read/write user's password
        :type password: str

        :param db: the name of the database to connect to
        :type db: str
        """
        # store vars
        self._host = host
        self._port = port
        self._username = username
        self._password = password
        self._db = db
        self.series = series

        # create client
        self.client = InfluxDBClient(self._host, self._port, self._username, self._password, self._db)

    def get_content(self, time_offset, limit=None):
        """gets trending content


        :param time_offset: the influxdb formatted time offset interval
        :type time_offset: str

        :param limit: a hard limit on the number of pieces of content returned
        :type limit: int or None
        :default limit: None

        :return: a list of content
        :rtype: [Content, ]
        """
        # construct query
        query = 'select sum(clicks) as clicks ' \
                'from {} ' \
                'where time > now() - {} ' \
                'group by content_id'\
            .format(self.series, time_offset)

        # execute query
        try:
            results = self.client.query(query)
        except:
            query = 'select * from {} where time > now() - {}'.format(self.series, time_offset)
            results = self.client.query(query)

        # make content objects
        content_objects = []
        for result in results:
            site = result.get('name', '').split('.')[0]
            columns = result.get('columns', [])
            points = result.get('points', [])
            for point in points:
                content = Content.from_query_result(columns, point)
                content.site = site
                content_objects.append(content)

        # sort content objects
        content_objects.sort(reverse=True)

        # return objects
        if limit:
            return content_objects[:limit]
        return content_objects



from distutils.core import setup


setup(
    name='influx-content-client',
    version='0.0.4',

    packages=['influx_content_client'],

    install_requires=[
        'influxdb==0.1.13'
    ],

    author='Vince Forgione',
    author_email='vforgione@theonion.com',
    description='a thin wrapper around the influxdb client to work with content series '
                '-- see https://github.com/theonion/influx-trending',

    url='https://github.com/theonion/influx-content-client'
)
